import { afterEach, beforeEach, describe, expect, it, vi } from 'vitest';
import shell from 'shelljs';
import PouchDB from 'pouchdb';
import { getTrkseg, getTrksegDocs, putTrkseg } from './trkseg';

import { putTrkpt } from './trkpt';
import { get, getFamily, put } from './lib';
import { emptyWpt } from './wpt';

const test = it;
const jest = vi;

declare global {
  var db: any;
  var dbReady: boolean;
}

const originalDb = globalThis.db;
const originalDateNow = globalThis.Date.now;

describe('The trkpt module with a real db', () => {
  beforeEach(async () => {
    globalThis.db = new PouchDB('dyomedea', {
      auto_compaction: false,
    });
    globalThis.Date.now = () => 0;
    // vi.mock('./rte', async () => {
    //   const originalModule = await vi.importActual('./rte');
    //   return {
    //     ...originalModule,
    //     compactTrksegOrTrksegDebounced: () => {
    //       console.log('mocked compactTrksegOrTrksegDebounced');
    //     },
    //   };
    // });
  });
  afterEach(async () => {
    try {
      await db.destroy();
    } catch (err) {
      //      console.error(err);
      await shell.exec('rm -rf  $*$');
    }
    globalThis.db = undefined;
    globalThis.dbReady = false;
    globalThis.Date.now = originalDateNow;
  });
  it('can write a trkpt inside a trkseg', async () => {
    const idTrkseg = 'gpx/4320836410265485/2trkseg/000034';
    const trkpt: Wpt = {
      $: { lat: 2, lon: 2 },
    };
    const trkseg0: Trkseg = {
      trkpt: [
        {
          $: { lat: 0, lon: 0 },
        },
        {
          $: { lat: 1, lon: 1 },
        },
      ],
    };
    const trkseg1: Trkseg = {
      trkpt: [
        {
          $: { lat: 0, lon: 0 },
        },
        {
          $: { lat: 2, lon: 2 },
        },
      ],
    };

    const trksegPut = await putTrkseg(idTrkseg, trkseg0);
    expect(trksegPut).toEqual(idTrkseg);
    const trksegPutTrksegpt = await putTrkpt({
      id: idTrkseg,
      index: 1,
      wpt: trkpt,
    });
    const trksegGetTrksegpt = await getTrkseg({ id: idTrkseg });
    expect(trksegGetTrksegpt).toEqual(trkseg1);
  });
  it('can write a trkpt outside a trkseg', async () => {
    const idTrkseg = 'gpx/4320836410265485/2trkseg/000034';
    const idTrksegpt = 'gpx/4320836410265485/2trkseg/000034/000035';
    const trkpt3: Wpt = {
      $: { lat: 3, lon: 3 },
    };
    const trkpt4: Wpt = {
      $: { lat: 4, lon: 4 },
    };
    const trkseg0: Trkseg = {
      trkpt: [
        {
          $: { lat: 1, lon: 1 },
        },
        {
          $: { lat: 2, lon: 2 },
        },
      ],
    };
    const trkseg1: Trkseg = {
      trkpt: [
        {
          $: { lat: 1, lon: 1 },
        },
        {
          $: { lat: 2, lon: 2 },
        },
        trkpt3,
      ],
    };
    const trkseg2: Trkseg = {
      trkpt: [
        {
          $: { lat: 1, lon: 1 },
        },
        {
          $: { lat: 2, lon: 2 },
        },
        trkpt4,
      ],
    };

    const trksegPut = await putTrkseg(idTrkseg, trkseg0);
    expect(trksegPut).toEqual(idTrkseg);
    expect(await getTrkseg({ id: idTrkseg })).toEqual(trkseg0);
    await put(idTrksegpt, 'trkpt', () => trkpt3, emptyWpt);
    const { trkseg, docs } = await getTrksegDocs({ id: idTrkseg });
    console.log({
      caller: 'test',
      trkptIn: JSON.stringify(docs.rows[0].doc.doc.trkpt),
    });
    expect(await getTrkseg({ id: idTrkseg })).toEqual(trkseg1);
    await putTrkpt({
      id: idTrkseg,
      index: 2,
      wpt: trkpt4,
    });
    expect((await getFamily(idTrkseg)).rows.length).toEqual(2);
    expect(await getTrkseg({ id: idTrkseg })).toEqual(trkseg2);
    expect((await get(idTrksegpt)).doc).toEqual(trkpt4);
  });
    it('can write a trkpt outside a trkseg with a negative index', async () => {
      const idTrkseg = 'gpx/4320836410265485/2trkseg/000034';
      const idTrksegpt = 'gpx/4320836410265485/2trkseg/000034/000035';
      const trkpt3: Wpt = {
        $: { lat: 3, lon: 3 },
      };
      const trkpt4: Wpt = {
        $: { lat: 4, lon: 4 },
      };
      const trkseg0: Trkseg = {
        trkpt: [
          {
            $: { lat: 1, lon: 1 },
          },
          {
            $: { lat: 2, lon: 2 },
          },
        ],
      };
      const trkseg1: Trkseg = {
        trkpt: [
          {
            $: { lat: 1, lon: 1 },
          },
          {
            $: { lat: 2, lon: 2 },
          },
          trkpt3,
        ],
      };
      const trkseg2: Trkseg = {
        trkpt: [
          {
            $: { lat: 1, lon: 1 },
          },
          {
            $: { lat: 2, lon: 2 },
          },
          trkpt4,
        ],
      };

      const trksegPut = await putTrkseg(idTrkseg, trkseg0);
      await put(idTrksegpt, 'trkpt', () => trkpt3, emptyWpt);
      await putTrkpt({
        id: idTrkseg,
        index: -1,
        wpt: trkpt4,
      });
      expect(await getTrkseg({ id: idTrkseg })).toEqual(trkseg2);
      expect((await get(idTrksegpt)).doc).toEqual(trkpt4);
    });
});
